@@ -5,15 +5,18 @@ use reqwest::header::ACCEPT;
 pub mod events;
 mod file;
 pub mod forks;
+mod pulls;
 pub mod releases;
 mod status;
 mod tags;
 mod generate;
 
 use crate::{models, params, Octocrab, Result};
+pub use file::GetContentBuilder;
 pub use file::UpdateFileBuilder;
+pub use pulls::ListPullsBuilder;
 pub use releases::ReleasesHandler;
-pub use status::CreateStatusBuilder;
+pub use status::{CreateStatusBuilder, ListStatusesBuilder};
 pub use tags::ListTagsBuilder;
 pub use generate::GenerateRepositoryBuilder;
 
@@ -145,6 +148,24 @@ impl<'octo> RepoHandler<'octo> {
             .await
     }
 
+    /// Get repository content.
+    /// ```no_run
+    /// # async fn run() -> octocrab::Result<()> {
+    ///
+    /// octocrab::instance()
+    ///     .repos("owner", "repo")
+    ///     .get_content()
+    ///     .path("path/to/file")
+    ///     .r#ref("main")
+    ///     .send()
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn get_content(&self) -> GetContentBuilder<'_, '_> {
+        GetContentBuilder::new(self)
+    }
+
     /// Creates a new file in the repository.
     /// ```no_run
     /// # async fn run() -> octocrab::Result<()> {
@@ -252,6 +273,16 @@ impl<'octo> RepoHandler<'octo> {
         CreateStatusBuilder::new(self, sha, state)
     }
 
+    /// List statuses for a reference.
+    pub fn list_statuses(&self, sha: String) -> ListStatusesBuilder<'_, '_> {
+        ListStatusesBuilder::new(self, sha)
+    }
+
+    /// List pull requests for a reference.
+    pub fn list_pulls(&self, sha: String) -> ListPullsBuilder<'_, '_> {
+        ListPullsBuilder::new(self, sha)
+    }
+
     /// List events on this repository.
     ///
     /// Takes an optional etag which allows for efficient polling. Here is a quick example to poll a
