@@ -1,4 +1,5 @@
 use super::*;
+use chrono::{DateTime, Utc};
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[non_exhaustive]
@@ -40,7 +41,9 @@ pub struct Run {
     pub cancel_url: Url,
     pub rerun_url: Url,
     pub workflow_url: Url,
-    pub pull_requests: Vec<super::pulls::PullRequest>,
+    // This is not correct! E.g. I see stuff that is only a subset of PullRequest, e.g.
+    // "pull_requests":[{"url":"https://api.github.com/repos/artichoke/artichoke/pulls/1346","id":717179206,"number":1346,"head":{"ref":"pernosco-integration","sha":"9ee4335ecfc3e7abe44bddadb117a23d0d63e4ee","repo":{"id":199196552,"url":"https://api.github.com/repos/artichoke/artichoke","name":"artichoke"}},"base":{"ref":"trunk","sha":"abbb7cf0c75ab51b84309ac547c3c3c089dd36eb","repo":{"id":199196552,"url":"https://api.github.com/repos/artichoke/artichoke","name":"artichoke"}}}]
+    // pub pull_requests: Vec<super::pulls::PullRequest>,
     // TODO: other attrs
     // ref: https://docs.github.com/en/rest/reference/actions#list-workflow-runs
     pub head_commit: HeadCommit,
@@ -70,7 +73,8 @@ pub struct Job {
     pub status: String,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub conclusion: Option<String>,
-    pub started_at: chrono::DateTime<chrono::Utc>,
+    // Github has been seen to set null here during Job startup
+    pub started_at: Option<chrono::DateTime<chrono::Utc>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub completed_at: Option<chrono::DateTime<chrono::Utc>>,
     pub name: String,
@@ -89,7 +93,23 @@ pub struct Step {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub conclusion: Option<String>,
     pub number: i64,
-    pub started_at: chrono::DateTime<chrono::Utc>,
+    // Github might set null here during Step startup...
+    pub started_at: Option<chrono::DateTime<chrono::Utc>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub completed_at: Option<chrono::DateTime<chrono::Utc>>,
 }
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[non_exhaustive]
+pub struct WorkflowListArtifact {
+    pub id: crate::models::ArtifactId,
+    pub node_id: String,
+    pub name: String,
+    pub size_in_bytes: usize,
+    pub url: Url,
+    pub archive_download_url: Url,
+    pub expired: bool,
+    pub created_at: DateTime<Utc>,
+    pub updated_at: DateTime<Utc>,
+    pub expires_at: DateTime<Utc>,
+}
